﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using HFLib.Common;
using System.Collections.Generic;

namespace HFLib.Controls
{
    public class HFColorPicker : Control
    {
        //////////////////////////////////////////////////////////////////////////////
        // UI

        Canvas cvMyself;

        // 기본
        public event EventHandler ColorChanged;
        enum RGB { Red, Green, Blue };

        // 색상표시
        Rectangle rcInit;
        Rectangle rcCurrent;
        Rectangle rcLast;

        // 글자
        TextBlock tbRedAmount;
        TextBlock tbGreenAmount;
        TextBlock tbBlueAmount;
        TextBlock tbAlphaAmount;
        TextBlock tbColorCode;

        // 색상 농도 바
        Rectangle rcRedCurrent;
        Rectangle rcGreenCurrent;
        Rectangle rcBlueCurrent;
        Rectangle rcAlphaCurrent;

        // 색상 조절기
        Canvas cvRedControl;
        Canvas cvGreenControl;
        Canvas cvBlueControl;
        Canvas cvAlphaControl;
        Canvas cvRed;
        Canvas cvGreen;
        Canvas cvBlue;
        Canvas cvAlpha;

        // 그라디언트
        GradientStop gstopRedLeft;
        GradientStop gstopRedRight;
        GradientStop gstopGreenLeft;
        GradientStop gstopGreenRight;
        GradientStop gstopBlueLeft;
        GradientStop gstopBlueRight;
        Rectangle rcAlphaGradient;

        // cvPickZone
        Canvas cvPickZone;
        Canvas cvColorCursor;
        GradientStop gstopPickZone;

        // Mother Color
        Canvas cvMotherColorCursor;
        Canvas cvMotherColorBar;
        int nMotherColorIndex = 0;
        List<Color> MotherColorList = new List<Color>();
        enum MotherColorGroup { RGB, GRB, GBR, BGR, BRG, RBG }
        int[] MotherColorGroupIndex = new int[__MOTHERGROUPCOUNT];
        
        // 기본
        bool bClicked = false;

        //////////////////////////////////////////////////////////////////////////////
        // 상수
        const double __COLORBARWIDTH = 70;
        const int __MAX_COLORAMOUNT = 255;
        const double __MAXPICKZONEX = 128;
        const double __MAXPICKZONEY = 128;
        const int __MAXFULLCOLOR_INDEX = 127;
        const int __MOTHERGROUPCOUNT = 7;

        //////////////////////////////////////////////////////////////////////////////
        // 자료구조

        // 색상
        Color _colorLast = Colors.Black;
        Color _color;
        Color _colorMother;
        Color _colorInitial;
        public Color Color { get { return _color; } set { SetColor(value); } }
        public Color ColorAlphaless { get { return GetColorAlphaless(_color); } }
        private Color MotherColorAlphaless { get { return GetColorAlphaless(_colorMother); } }
        private Color LastColor { get { return GetColorAlphaless(_colorLast); } }

        // UI 위치
        private Point _Position = new Point();
        public Point Position { get { return _Position; } set { SetPosition(value); } }

        Point ptColorCursor = new Point(128, 0);

        public HFColorPicker()
        {
            System.IO.Stream s = this.GetType().Assembly.GetManifestResourceStream("HFLib.Controls.HFColorPicker.xaml");
            cvMyself = this.InitializeFromXaml(new System.IO.StreamReader(s).ReadToEnd()) as Canvas;

            // User Control 사이즈
            Width = cvMyself.Width;
            Height = cvMyself.Height;

            // 하단 현재 색상, 이전 색상 표시
            rcInit = cvMyself.FindName("rcInit") as Rectangle;
            rcCurrent = cvMyself.FindName("rcCurrent") as Rectangle;
            rcLast = cvMyself.FindName("rcLast") as Rectangle;

            // 색상의 양 (R, G, B, Alpha)
            tbRedAmount = cvMyself.FindName("tbRedAmount") as TextBlock;
            tbGreenAmount = cvMyself.FindName("tbGreenAmount") as TextBlock;
            tbBlueAmount = cvMyself.FindName("tbBlueAmount") as TextBlock;
            tbAlphaAmount = cvMyself.FindName("tbAlphaAmount") as TextBlock;

            // 색상 코드
            tbColorCode = cvMyself.FindName("tbColorCode") as TextBlock;

            // 색상 농도 바
            rcRedCurrent = cvMyself.FindName("rcRedCurrent") as Rectangle;
            rcGreenCurrent = cvMyself.FindName("rcGreenCurrent") as Rectangle;
            rcBlueCurrent = cvMyself.FindName("rcBlueCurrent") as Rectangle;
            rcAlphaCurrent = cvMyself.FindName("rcAlphaCurrent") as Rectangle;

            // 색상 조절기
            cvRedControl = cvMyself.FindName("cvRedControl") as Canvas;
            cvGreenControl = cvMyself.FindName("cvGreenControl") as Canvas;
            cvBlueControl = cvMyself.FindName("cvBlueControl") as Canvas;
            cvAlphaControl = cvMyself.FindName("cvAlphaControl") as Canvas;
            cvRed = cvMyself.FindName("cvRed") as Canvas;
            cvGreen = cvMyself.FindName("cvGreen") as Canvas;
            cvBlue = cvMyself.FindName("cvBlue") as Canvas;
            cvAlpha = cvMyself.FindName("cvAlpha") as Canvas;
            gstopRedLeft = cvMyself.FindName("gstopRedLeft") as GradientStop;
            gstopRedRight = cvMyself.FindName("gstopRedRight") as GradientStop;
            gstopGreenLeft = cvMyself.FindName("gstopGreenLeft") as GradientStop;
            gstopGreenRight = cvMyself.FindName("gstopGreenRight") as GradientStop;
            gstopBlueLeft = cvMyself.FindName("gstopBlueLeft") as GradientStop;
            gstopBlueRight = cvMyself.FindName("gstopBlueRight") as GradientStop;
            rcAlphaGradient = cvMyself.FindName("rcAlphaGradient") as Rectangle;

            // cvPickZone
            cvPickZone = cvMyself.FindName("cvPickZone") as Canvas;
            cvColorCursor = cvMyself.FindName("cvColorCursor") as Canvas;
            gstopPickZone = cvMyself.FindName("gstopPickZone") as GradientStop;

            // Mother Color
            cvMotherColorCursor = cvMyself.FindName("cvMotherColorCursor") as Canvas;
            cvMotherColorBar = cvMyself.FindName("cvMotherColorBar") as Canvas;

            ///////////////////////////////////////////////////////////////////////////////////////
            // 이벤트

            cvRed.MouseLeftButtonDown += new MouseEventHandler(cvRed_MouseLeftButtonDown);
            cvRed.MouseLeftButtonUp += new MouseEventHandler(cvRed_MouseLeftButtonUp);
            cvRed.MouseMove += new MouseEventHandler(cvRed_MouseMove);

            cvGreen.MouseLeftButtonDown += new MouseEventHandler(cvGreen_MouseLeftButtonDown);
            cvGreen.MouseLeftButtonUp += new MouseEventHandler(cvGreen_MouseLeftButtonUp);
            cvGreen.MouseMove += new MouseEventHandler(cvGreen_MouseMove);

            cvBlue.MouseLeftButtonDown += new MouseEventHandler(cvBlue_MouseLeftButtonDown);
            cvBlue.MouseLeftButtonUp += new MouseEventHandler(cvBlue_MouseLeftButtonUp);
            cvBlue.MouseMove += new MouseEventHandler(cvBlue_MouseMove);

            cvAlpha.MouseLeftButtonDown += new MouseEventHandler(cvAlpha_MouseLeftButtonDown);
            cvAlpha.MouseLeftButtonUp += new MouseEventHandler(cvAlpha_MouseLeftButtonUp);
            cvAlpha.MouseMove += new MouseEventHandler(cvAlpha_MouseMove);

            cvPickZone.MouseLeftButtonDown += new MouseEventHandler(cvPickZone_MouseLeftButtonDown);
            cvPickZone.MouseLeftButtonUp += new MouseEventHandler(cvPickZone_MouseLeftButtonUp);
            cvPickZone.MouseMove += new MouseEventHandler(cvPickZone_MouseMove);

            rcInit.MouseLeftButtonUp += new MouseEventHandler(rcInit_MouseLeftButtonUp);
            rcLast.MouseLeftButtonUp += new MouseEventHandler(rcLast_MouseLeftButtonUp);

            cvMotherColorBar.MouseLeftButtonDown += new MouseEventHandler(cvMotherColorBar_MouseLeftButtonDown);
            cvMotherColorBar.MouseLeftButtonUp += new MouseEventHandler(cvMotherColorBar_MouseLeftButtonUp);
            cvMotherColorBar.MouseMove += new MouseEventHandler(cvMotherColorBar_MouseMove);

            MouseLeave += new EventHandler(HFColorPicker_MouseLeave);
        }

        void rcInit_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            CopyColorExceptAlpha(_colorInitial, ref _color);
            Update();
        }

        protected virtual void OnColorChanged(EventArgs e)
        {
            if (ColorChanged != null)
            {
                ColorChanged(this, e);
            }
        }

        void cvMotherColorBar_MouseMove(object sender, MouseEventArgs e)
        {
            if (bClicked == false)
                return;

            nMotherColorIndex = Math.Min((int)e.GetPosition(cvMotherColorBar).Y, __MAXFULLCOLOR_INDEX);

            // 현재 Mother 색상 구하기
            CopyColorExceptAlpha(MotherColorList[nMotherColorIndex], ref _colorMother);

            // 현재 색상 구하기
            CopyColorExceptAlpha(GetColorByPickPosition(ptColorCursor), ref _color);

            Update();            
        }

        void cvMotherColorBar_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            bClicked = false;
            cvMotherColorBar.ReleaseMouseCapture();
        }

        
        void cvMotherColorBar_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            bClicked = true;
            cvMotherColorBar.CaptureMouse();

            nMotherColorIndex = Math.Min((int)e.GetPosition(cvMotherColorBar).Y, __MAXFULLCOLOR_INDEX);

            // 현재 Mother 색상 구하기
            CopyColorExceptAlpha(MotherColorList[nMotherColorIndex], ref _colorMother);

            // 현재 색상 구하기
            CopyColorExceptAlpha(GetColorByPickPosition(ptColorCursor), ref _color);

            Update();            
        }

        void cvPickZone_MouseMove(object sender, MouseEventArgs e)
        {
            if (bClicked == false)
                return;

            ptColorCursor = HFUtility.MarginCutter(e.GetPosition(cvPickZone), cvPickZone);

            // 현재 색상 구하기
            CopyColorExceptAlpha(GetColorByPickPosition(ptColorCursor), ref _color);

            Update();
        }

        void cvPickZone_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            bClicked = false;
            cvPickZone.ReleaseMouseCapture();

            _colorLast = _color;
            Update();
        }

        void cvPickZone_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            bClicked = true;
            cvPickZone.CaptureMouse();

            ptColorCursor = HFUtility.MarginCutter(e.GetPosition(cvPickZone), cvPickZone);

            // 현재 색상 구하기
            CopyColorExceptAlpha(GetColorByPickPosition(ptColorCursor), ref _color);

            Update();
        }

        void rcLast_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            CopyColorExceptAlpha(_colorLast, ref _color);
            Update();
        }

        void CopyColorExceptAlpha(Color colorFrom, ref Color colorTo)
        {
            // Alpha 제외 카피
            colorTo.R = colorFrom.R;
            colorTo.G = colorFrom.G;
            colorTo.B = colorFrom.B;
        }

        void HFColorPicker_MouseLeave(object sender, EventArgs e)
        {
            bClicked = false;
        }

        #region "Red Control"
        void cvRed_MouseMove(object sender, MouseEventArgs e)
        {
            if (bClicked == false)
                return;

            Point ptCurrent = e.GetPosition(cvRedControl);
            _color.R = (byte)GetColorAmountByColorBar(ptCurrent.X);

            CorrectCursors();
            Update();
        }

        void cvRed_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            cvRed.ReleaseMouseCapture();
            bClicked = false;
        }

        void cvRed_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            cvRed.CaptureMouse();
            bClicked = true;

            Point ptCurrent = e.GetPosition(cvRedControl);
            _color.R = (byte)GetColorAmountByColorBar(ptCurrent.X);

            CorrectCursors();
            Update();
        }
        #endregion

        #region "Green Control"
        void cvGreen_MouseMove(object sender, MouseEventArgs e)
        {
            if (bClicked == false)
                return;

            Point ptCurrent = e.GetPosition(cvGreenControl);
            _color.G = (byte)GetColorAmountByColorBar(ptCurrent.X);

            CorrectCursors();
            Update();
        }

        void cvGreen_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            cvGreen.ReleaseMouseCapture();
            bClicked = false;
        }

        void cvGreen_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            cvGreen.CaptureMouse();
            bClicked = true;

            Point ptCurrent = e.GetPosition(cvGreenControl);
            _color.G = (byte)GetColorAmountByColorBar(ptCurrent.X);

            CorrectCursors();
            Update();
        }
        #endregion

        #region "Blue Control"
        void cvBlue_MouseMove(object sender, MouseEventArgs e)
        {
            if (bClicked == false)
                return;

            Point ptCurrent = e.GetPosition(cvBlueControl);
            _color.B = (byte)GetColorAmountByColorBar(ptCurrent.X);

            CorrectCursors();
            Update();
        }

        void cvBlue_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            cvBlue.ReleaseMouseCapture();
            bClicked = false;
        }

        void cvBlue_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            cvBlue.CaptureMouse();
            bClicked = true;

            Point ptCurrent = e.GetPosition(cvBlueControl);
            _color.B = (byte)GetColorAmountByColorBar(ptCurrent.X);

            CorrectCursors();
            Update();
        }
        #endregion

        #region "Alpha Control"
        void cvAlpha_MouseMove(object sender, MouseEventArgs e)
        {
            if (bClicked == false)
                return;

            Point ptCurrent = e.GetPosition(cvAlphaControl);
            _color.A = (byte)GetColorAmountByColorBar(ptCurrent.X);

            Update();
        }

        void cvAlpha_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            cvAlpha.ReleaseMouseCapture();
            bClicked = false;
        }

        void cvAlpha_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            cvAlpha.CaptureMouse();
            bClicked = true;

            Point ptCurrent = e.GetPosition(cvAlphaControl);
            _color.A = (byte)GetColorAmountByColorBar(ptCurrent.X);

            Update();
        }

        #endregion

        private int GetColorAmountByColorBar(double dPosition)
        {
            if (dPosition < 0)
                dPosition = 0;

            if (dPosition > __COLORBARWIDTH)
                dPosition = __COLORBARWIDTH;

            int nAmount = (int)((double)__MAX_COLORAMOUNT * dPosition / __COLORBARWIDTH);

            return nAmount;
        }

        private void SetColor(Color color)
        {
            if (color == null)
                return;

            // 초기 색상 백업
            _colorInitial = color;

            // 자료구조 업데이트
            _color = color;

            // Mother Color 설정
            CorrectCursors();

            Update();
        }

        int FindMotherSpecial(Color color)
        {
            for (int i = 0; i < MotherColorList.Count; i++)
			{
                if (color == MotherColorList[i])
                    return i;
			}
            return -1;
        }

        int GetBestMotherColorIndex(Color color)
        {
            ///////////////////////////////////////////////////////////////////////////
            // 정확히 같은 Mother Color가 있는가?
            int nFind = FindMotherSpecial(color);
            if (nFind != -1)
                return nFind;

            ///////////////////////////////////////////////////////////////////////////
            // 2색 이상 같은 값을 가질 경우 : 경계값

            if (CompareColorAmount(color.R, color.G) == true)
            {
                if (CompareColorAmount(color.R, color.B) == true) // R=G=B 무채색인 경우로 현재의 MotherColor를 유지
                    return nMotherColorIndex; 
                else if (color.B < color.R)
                    return MotherColorGroupIndex[(int)MotherColorGroup.GRB];
                else
                    return MotherColorGroupIndex[(int)MotherColorGroup.BRG];
            }
            else if (CompareColorAmount(color.G, color.B))
            {
                if (CompareColorAmount(color.G, color.R) == true) // R=G=B 무채색인 경우로 현재의 MotherColor를 유지
                    return nMotherColorIndex;
                else if (color.R < color.G)
                    return MotherColorGroupIndex[(int)MotherColorGroup.BGR];
                else
                    return MotherColorGroupIndex[(int)MotherColorGroup.RGB];
            }
            else if (CompareColorAmount(color.B, color.R))
            {
                if (CompareColorAmount(color.B, color.G) == true) // R=G=B 무채색인 경우로 현재의 MotherColor를 유지
                    return nMotherColorIndex;
                else if (color.G < color.B)
                    return MotherColorGroupIndex[(int)MotherColorGroup.RBG];
                else
                    return MotherColorGroupIndex[(int)MotherColorGroup.GBR];
            }
           
            ///////////////////////////////////////////////////////////////////////////
            // Step 1. Red, Green, Blue 순서 배열하여 MotherGroup 찾기

            MotherColorGroup MotherGroup = MotherColorGroup.RGB;

            if (color.R > color.G)
            {
                if (color.R > color.B)
                {
                    if (color.B > color.G)
                    {
                        MotherGroup = MotherColorGroup.RBG;
                    }
                    else
                    {
                        MotherGroup = MotherColorGroup.RGB;
                    }
                }
                else
                {
                    MotherGroup = MotherColorGroup.BRG;
                }
            }
            else
            {
                if (color.G > color.B)
                {
                    if (color.R > color.B)
                    {
                        MotherGroup = MotherColorGroup.GRB;
                    }
                    else
                    {
                        MotherGroup = MotherColorGroup.GBR;
                    }
                }
                else
                {
                    MotherGroup = MotherColorGroup.BGR;
                }
            }

            ///////////////////////////////////////////////////////////////////////////
            // 1개가 0인경우

            int nTemp = MotherColorGroupIndex[(int)MotherGroup];
            Color colorCopy = color;

            if (color.R == 0)
            {
                double dMultiple = 255.0 / Math.Max(color.B, color.G);
                colorCopy.G = (byte)((double)color.G * dMultiple);
                colorCopy.B = (byte)((double)color.B * dMultiple);

                if (CompareColorAmount(colorCopy.G, 255))
                {
                    int nIndex = nTemp + (int)(Math.Ceiling((double)colorCopy.B / (double)__MAX_COLORAMOUNT * 21.0));
                    return nIndex;
                }
                else if (CompareColorAmount(colorCopy.B, 255))
                {
                    int nIndex = nTemp + (21 - (int)(Math.Ceiling((double)colorCopy.G / (double)__MAX_COLORAMOUNT * 21.0)));
                    return nIndex;
                }
            }
            else if (color.G == 0)
            {
                double dMultiple = 255.0 / Math.Max(color.B, color.R);
                colorCopy.B = (byte)((double)color.B * dMultiple);
                colorCopy.R = (byte)((double)color.R * dMultiple);

                if (CompareColorAmount(colorCopy.R, 255))
                {
                    int nIndex = nTemp + (21 - (int)(Math.Ceiling((double)colorCopy.B / (double)__MAX_COLORAMOUNT * 21.0)));
                    return nIndex;
                }
                else if (CompareColorAmount(colorCopy.B, 255))
                {
                    int nIndex = nTemp + (int)(Math.Ceiling((double)colorCopy.R / (double)__MAX_COLORAMOUNT * 21.0));
                    return nIndex;
                }
            }
            else if (color.B == 0)
            {
                double dMultiple = 255.0 / Math.Max(color.R, color.G);
                colorCopy.R = (byte)((double)color.R * dMultiple);
                colorCopy.G = (byte)((double)color.G * dMultiple);

                if (CompareColorAmount(colorCopy.R, 255))
                {
                    int nIndex = nTemp + (int)(Math.Ceiling((double)colorCopy.G / (double)__MAX_COLORAMOUNT * 21.0));
                    return nIndex;
                }
                else if (CompareColorAmount(colorCopy.G, 255))
                {
                    int nIndex = nTemp + (21 - (int)(Math.Ceiling((double)colorCopy.R / (double)__MAX_COLORAMOUNT * 21.0)));
                    return nIndex;
                }
            }

            switch (MotherGroup)
            {
                case MotherColorGroup.RGB:
                case MotherColorGroup.GRB:
                case MotherColorGroup.GBR:
                case MotherColorGroup.BGR:
                case MotherColorGroup.BRG:
                case MotherColorGroup.RBG:
                    {
                        double dMin = double.MaxValue;
                        int nMinIndex = 0;

                        for (int i = 0; i < 21; i++)
                        {
                            int nIndexTemp = MotherColorGroupIndex[(int)MotherGroup] + i;
                            Color colorTemp = MotherColorList[nIndexTemp];

                            for (int j = 0; j <= 128; j++)
                            {
                                double dR = colorTemp.R + (double)(255 - colorTemp.R) * (128 - j) / 128;
                                double dG = colorTemp.G + (double)(255 - colorTemp.G) * (128 - j) / 128;
                                double dB = colorTemp.B + (double)(255 - colorTemp.B) * (128 - j) / 128;

                                double dRatioR = dR / (double)color.R;
                                double dRatioG = dG / (double)color.G;
                                double dRatioB = dB / (double)color.B;
                                double dAvg = (dRatioR + dRatioG + dRatioB) / 3.0;

                                double dFar = Math.Sqrt(Math.Pow(dRatioR - dAvg, 2) + Math.Pow(dRatioG - dAvg, 2) + Math.Pow(dRatioB - dAvg, 2));

                                if (dFar < dMin)
                                {
                                    dMin = dFar;
                                    nMinIndex = nIndexTemp;
                                }
                            }
                        }

                        return nMinIndex;
                    }
            }

            return 0;
        }

        private void CorrectCursors()
        {
            // Mother Color Cursor 위치 수정
            nMotherColorIndex = GetBestMotherColorIndex(_color);

            if ((_color.R == __MAX_COLORAMOUNT || _color.G == __MAX_COLORAMOUNT || _color.B == __MAX_COLORAMOUNT)
                && _color.R * _color.G * _color.B == 0)
            {
                _colorMother = _color;
            }
            else
            {
                _colorMother = MotherColorList[nMotherColorIndex];
            }

            // cvPickZone Color Cursor 위치 수정
            if (CompareColor(_colorMother, _color) == true) // 현재 색상이 Mother Color와 거의 같은 경우
            {
                ptColorCursor.X = 128;
                ptColorCursor.Y = 0;
            }
            else if (IsGraytone(_color))
            {
                ptColorCursor.X = 0;
                ptColorCursor.Y = Math.Ceiling((255 - _color.R) / 2);
            }
            else
            {
                // x 찾기
                double dFindR = 0;
                double dFindG = 0;
                double dFindB = 0;
                double dGapMin = double.MaxValue;
                for (int i = 0; i <= 128; i++)
                {
                    double dR = _colorMother.R + (__MAX_COLORAMOUNT - _colorMother.R) * (128 - i) / 128;
                    double dG = _colorMother.G + (__MAX_COLORAMOUNT - _colorMother.G) * (128 - i) / 128;
                    double dB = _colorMother.B + (__MAX_COLORAMOUNT - _colorMother.B) * (128 - i) / 128;

                    int nCount = 0;
                    double dRatioR = 0.0;
                    double dRatioG = 0.0;
                    double dRatioB = 0.0;

                    double dSum = 0.0;

                    if (dR != 0)
                    {
                        dRatioR = _color.R / dR;
                        nCount++;
                        dSum += dRatioR;
                    }

                    if (dG != 0)
                    {
                        dRatioG = _color.G / dG;
                        nCount++;
                        dSum += dRatioG;
                    }

                    if (dB != 0)
                    {
                        dRatioB = _color.B / dB;
                        nCount++;
                        dSum += dRatioB;
                    }

                    double dAvg = (dRatioR + dRatioG + dRatioB) / (double)nCount;

                    double dGapTemp = 0.0;

                    if (_color.R != 0)
                        dGapTemp += Math.Pow(dRatioR - dAvg, 2);
                    if (_color.G != 0)
                        dGapTemp += Math.Pow(dRatioG - dAvg, 2);
                    if (_color.B != 0)
                        dGapTemp += Math.Pow(dRatioB - dAvg, 2);

                    double dGap = Math.Sqrt(dGapTemp);

                    if (dGap < dGapMin)
                    {
                        dGapMin = dGap;
                        ptColorCursor.X = i;

                        dFindR = dR;
                        dFindG = dG;
                        dFindB = dB;
                    }
                }

                // y 찾기
                if (dFindR != 0 && _color.R != 0)
                    ptColorCursor.Y = Math.Max(0, 128 * (dFindR - (double)_color.R) / dFindR);
                else if (dFindG != 0 && _color.G != 0)
                    ptColorCursor.Y = Math.Max(0,  128 * (dFindG - (double)_color.G) / dFindG);
                else if (dFindB != 0 && _color.B != 0)
                    ptColorCursor.Y = Math.Max(0, 128 * (dFindB - (double)_color.B) / dFindB);
                else
                    ptColorCursor.Y = 128;
            }
        }

        bool CompareColor(Color color1, Color color2)
        {
            if (CompareColorAmount(color1.R, color2.R) == true &&
                CompareColorAmount(color1.G, color2.G) == true &&
                CompareColorAmount(color1.B, color2.B) == true)
                return true;

            return false;
        }

        bool CompareColorAmount(byte A, byte B)
        {
            if (Math.Abs(A - B) <= 1)
            {
                return true;
            }

            return false;
        }

        bool IsGraytone(Color color)
        {
            if (CompareColorAmount(color.R, color.G) &&
                CompareColorAmount(color.G, color.B) &&
                CompareColorAmount(color.B, color.R))
                return true;

            return false;
        }

        private void Update()
        {
            // 커서 위치
            UpdateCursorPosition();

            // 알파값 없는 색상준비
            Color colorNoAlpha = _color;
            colorNoAlpha.A = 0xFF;

            // 마지막 색상 업데이트
            rcLast.Fill = new SolidColorBrush(LastColor);

            // 현재 색상 업데이트
            rcInit.Fill = new SolidColorBrush(_colorInitial);
            rcCurrent.Fill = new SolidColorBrush(_color);

            tbRedAmount.Text = ((int)_color.R).ToString();
            tbGreenAmount.Text = ((int)_color.G).ToString();
            tbBlueAmount.Text = ((int)_color.B).ToString();

            int nAlpha = (int)(Math.Ceiling((double)_color.A / (double)__MAX_COLORAMOUNT * 100));
            tbAlphaAmount.Text = string.Format("{0}%", nAlpha);

            // cvPickZone 그라데이션
            gstopPickZone.Color = MotherColorAlphaless;
            UpdateGradient();

            // 색상 농도 바
            UpdateColorBars(_color);

            // 색상코드
            UpdateColorCode();

            OnColorChanged(null);
        }

        void UpdateCursorPosition()
        {
            cvColorCursor.SetValue(Canvas.LeftProperty, ptColorCursor.X);
            cvColorCursor.SetValue(Canvas.TopProperty, ptColorCursor.Y);

            cvMotherColorCursor.SetValue(Canvas.TopProperty, (double)nMotherColorIndex);
        }

        void UpdateGradient()
        {
            Color color;

            // Red
            color = this.ColorAlphaless;

            color.R = 0;
            gstopRedLeft.Color = color;

            color.R = 0xFF;
            gstopRedRight.Color = color;

            // Green
            color = this.ColorAlphaless;

            color.G = 0;
            gstopGreenLeft.Color = color;

            color.G = 0xFF;
            gstopGreenRight.Color = color;

            // Blue
            color = this.ColorAlphaless;

            color.B = 0;
            gstopBlueLeft.Color = color;

            color.B = 0xFF;
            gstopBlueRight.Color = color;

            // Alpha
            rcAlphaGradient.Fill = new SolidColorBrush(this.ColorAlphaless);
        }

        Color GetColorAlphaless(Color color)
        {
            Color colorTemp = new Color();
            CopyColorExceptAlpha(color, ref colorTemp);
            colorTemp.A = 0xFF;

            return colorTemp;
        }

        void UpdateColorCode()
        {
            tbColorCode.Text = string.Format("#{0:X2}{1:X2}{2:X2}{3:X2}",

                _color.A,
                _color.R,
                _color.G,
                _color.B);
        }

        void UpdateColorBars(Color color)
        {
            UpdateColorBar(rcRedCurrent, (int)color.R);
            UpdateColorBar(rcGreenCurrent, (int)color.G);
            UpdateColorBar(rcBlueCurrent, (int)color.B);
            UpdateColorBar(rcAlphaCurrent, (int)color.A);
        }

        void UpdateColorBar(Rectangle rc, int nColorAmount)
        {
            double dWidth = ((double)nColorAmount / (double)255) * __COLORBARWIDTH;
            rc.Width = dWidth;
        }

        Color GetColorByPickPosition(Point pt)
        {
            Color color = MotherColorAlphaless;

            double dRatioX = (__MAXPICKZONEX - pt.X) / __MAXPICKZONEX;
            double dRatioY = (__MAXPICKZONEY - pt.Y) / __MAXPICKZONEY;

            color.R = IncreaseColorAmount(color.R, dRatioX);
            color.G = IncreaseColorAmount(color.G, dRatioX);
            color.B = IncreaseColorAmount(color.B, dRatioX);

            color.R = GetColorAmountByPropotion(color.R, dRatioY);
            color.G = GetColorAmountByPropotion(color.G, dRatioY);
            color.B = GetColorAmountByPropotion(color.B, dRatioY);

            return color;
        }

        byte IncreaseColorAmount(byte byteColor, double dRatio)
        {
            int nTemp = Math.Min(__MAX_COLORAMOUNT, (int)(byteColor + (__MAX_COLORAMOUNT - byteColor) * dRatio));
            return (byte)nTemp;
        }

        byte GetColorAmountByPropotion(byte byteColor, double dRatio)
        {
            int nTemp = Math.Min(__MAX_COLORAMOUNT, (int)(byteColor * dRatio));
            return (byte)nTemp;
        }

        void SetPosition(Point pt)
        {
            SetValue(Canvas.LeftProperty, pt.X);
            SetValue(Canvas.TopProperty, pt.Y);
        }
    }
}
