﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Controls.Primitives;

namespace HugeFlow.Controls.ColorPicker
{
    public partial class ColorBox : UserControl, IColorChangable
    {
        #region Properties

        /// <summary>
        /// 색상
        /// </summary>
        public Color Color
        {
            get { return (rcColor.Fill as SolidColorBrush).Color; }
            set
            {
                if ((rcColor.Fill as SolidColorBrush).Color == value)
                    return;

                rcColor.Fill = new SolidColorBrush(value);

                OnColorChanged();
            }
        }
        #endregion Properties


        #region Events

        public event EventHandler ColorChanged;

        private void OnColorChanged()
        {
            if (ColorChanged == null)
                return;

            ColorChanged(this, null);
        }

        #endregion Events

        #region Constructors

        /// <summary>
        /// 기본 생성자
        /// </summary>
        public ColorBox()
        {
            InitializeComponent();
        }

        #endregion Constructors

        #region EventHandlers

        private void LayoutRoot_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            BlendColorPicker colorPicker = new BlendColorPicker();

            colorPicker.Show(Color, this);
        }

        #endregion EventHandlers

        #region IColorChangable Members

        public void UpdateColor(Color color)
        {
            Color = color;
        }

        #endregion

        #region IColorChangable Members

        public Point AbsolutePosition
        {
            get
            {
                FrameworkElement current = this.Parent as FrameworkElement;

                while (current.Parent != null)
                {
                    current = current.Parent as FrameworkElement;
                }


                GeneralTransform transform = this.TransformToVisual(current);

                return transform.Transform(new Point(15, 15));
            }
        }

        #endregion
    }
}
