﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace HugeFlow.Controls.ColorPicker
{
    public partial class ColorSpace : UserControl
    {
        #region Properties

        private const double _MAXPICKZONEX = 128;
        private const double _MAXPICKZONEY = 128;

        private Color _color;
        /// <summary>
        /// 색상
        /// </summary>
        public Color Color
        {
            get { return _color; }
            set
            {
                _color = value;

                UpdateCursorPositionByColor(_color);
            }
        }


        private Color _motherColor;
        public Color MotherColor
        {
            get { return _motherColor; }
            set
            {
                _motherColor = value;
                gstopPickZone.Color = _motherColor;

                // 부모 색상 변경에 따라 자동으로 색상을 변경해 줌
                _color = GetColorByPickPosition(_colorCursorPosition);
            }
        }

        #endregion Properties


        #region Events


        public event EventHandler PointMoved;

        private void OnPointMoved()
        {
            if (PointMoved == null)
                return;

            PointMoved(this, null);
        }


        #endregion Events

        #region Constructors

        /// <summary>
        /// 기본 생성자
        /// </summary>
        public ColorSpace()
        {
            // Required to initialize variables
            InitializeComponent();

            LinkEvents();
        }

        #endregion Constructors


        #region Methods


        private void UpdateCursorPositionByColor(Color color)
        {
            // 새터레이션
            double sat = ColorUtility.GetHSVSaturation(color);
            double light = ColorUtility.GetHSVLight(color);

            Point newPosition = new Point(sat * cvPickZone.Width, (1-light) * cvPickZone.Height);
            UpdateCursorPosition(newPosition);
        }

        private Color GetColorByPosition()
        {
            return GetColorByPickPosition(_colorCursorPosition);
        }

        /// <summary>
        /// 이벤트를 연결합니다.
        /// </summary>
        private void LinkEvents()
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="pt"></param>
        /// <returns></returns>
        private Color GetColorByPickPosition(Point pt)
        {
            Color color = ColorUtility.GetNetColor(MotherColor);

            double dRatioX = (_MAXPICKZONEX - pt.X) / _MAXPICKZONEX;
            double dRatioY = (_MAXPICKZONEY - pt.Y) / _MAXPICKZONEY;

            color.R = ColorUtility.IncreaseColorAmount(color.R, dRatioX);
            color.G = ColorUtility.IncreaseColorAmount(color.G, dRatioX);
            color.B = ColorUtility.IncreaseColorAmount(color.B, dRatioX);

            color.R = ColorUtility.GetColorAmountByPropotion(color.R, dRatioY);
            color.G = ColorUtility.GetColorAmountByPropotion(color.G, dRatioY);
            color.B = ColorUtility.GetColorAmountByPropotion(color.B, dRatioY);

            return color;
        }

        #endregion Methods

        #region EventHandlers
        void cvPickZone_MouseMove(object sender, MouseEventArgs e)
        {
            if (_isClicked == false)
                return;

            UpdateCursorPosition(e.GetPosition(cvPickZone));

            _color = GetColorByPickPosition(_colorCursorPosition);

            OnPointMoved();
        }

        void cvPickZone_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            e.Handled = true;

            _isClicked = false;
            cvPickZone.ReleaseMouseCapture();

            UpdateCursorPosition(e.GetPosition(cvPickZone));

            _color = GetColorByPickPosition(_colorCursorPosition);

            OnPointMoved();
        }

        private void UpdateCursorPosition(Point position)
        {
            _colorCursorPosition.X = Math.Max(0, Math.Min(position.X, cvPickZone.Width - 1));
            _colorCursorPosition.Y = Math.Max(0, Math.Min(position.Y, cvPickZone.Height - 1));

            cvColorCursor.SetValue(Canvas.LeftProperty, _colorCursorPosition.X);
            cvColorCursor.SetValue(Canvas.TopProperty, _colorCursorPosition.Y);
        }

        private bool _isClicked = false;
        private Point _colorCursorPosition = new Point(128, 0);

        void cvPickZone_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            e.Handled = true;

            _isClicked = true;
            cvPickZone.CaptureMouse();

            UpdateCursorPosition(e.GetPosition(cvPickZone));

            _color = GetColorByPickPosition(_colorCursorPosition);

            OnPointMoved();           
        }
        #endregion EventHandlers


    }
}