﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace HugeFlow.Controls.ColorPicker
{
    public class ColorUtility
    {
        static public byte IncreaseColorAmount(byte byteColor, double dRatio)
        {
            int nTemp = Math.Min((int)byte.MaxValue, (int)(byteColor + ((int)byte.MaxValue - byteColor) * dRatio));
            return (byte)nTemp;
        }

        static public byte GetColorAmountByPropotion(byte byteColor, double dRatio)
        {
            int nTemp = Math.Min((int)byte.MaxValue, (int)(byteColor * dRatio));
            return (byte)nTemp;
        }

        /// <summary>
        /// 알파를 제외한 색상 얻어오기
        /// </summary>
        /// <param name="color"></param>
        /// <returns></returns>
        static public Color GetNetColor(Color color)
        {
            Color netColor = color;
            netColor.A = 0xFF;

            return netColor;
        }

        static public bool CompareColorAmount(byte A, byte B)
        {
            if (Math.Abs(A - B) <= 1)
            {
                return true;
            }

            return false;
        }

        /// <summary>
        /// 색상값을 0~1사이의 값으로 단위화
        /// </summary>
        /// <param name="colorByte"></param>
        /// <returns></returns>
        static public double GetUnitColorAmount(byte colorByte)
        {
            return (double)colorByte / byte.MaxValue;
        }

        /// <summary>
        /// 색상의 밝기를 반환합니다.
        /// </summary>
        /// <param name="color"></param>
        /// <returns></returns>
        static public double GetHSLLight(Color color)
        {
            double r = GetUnitColorAmount(color.R);
            double g = GetUnitColorAmount(color.G);
            double b = GetUnitColorAmount(color.B);

            double max = Math.Max(Math.Max(r, g), b);
            double min = Math.Min(Math.Min(r, g), b);

            return (min + max) / 2;
        }

        static public double GetHSLSaturation(Color color)
        {
            double r = GetUnitColorAmount(color.R);
            double g = GetUnitColorAmount(color.G);
            double b = GetUnitColorAmount(color.B);

            double max = Math.Max(Math.Max(r, g), b);
            double min = Math.Min(Math.Min(r, g), b);

            if (max == min)
                return 0;

            double light = (min + max) / 2;

            if (light <= 0.5)
            {
                return (max - min) / (2 * light);
            }
            else
            {
                return (max - min) / (2 - 2 * light);
            }
        }

        /// <summary>
        /// 색상의 밝기를 반환합니다.
        /// </summary>
        /// <param name="color"></param>
        /// <returns></returns>
        static public double GetHSVLight(Color color)
        {
            double r = GetUnitColorAmount(color.R);
            double g = GetUnitColorAmount(color.G);
            double b = GetUnitColorAmount(color.B);

            double max = Math.Max(Math.Max(r, g), b);
            
            return max;
        }

        static public double GetHSVSaturation(Color color)
        {
            double r = GetUnitColorAmount(color.R);
            double g = GetUnitColorAmount(color.G);
            double b = GetUnitColorAmount(color.B);

            double max = Math.Max(Math.Max(r, g), b);
            double min = Math.Min(Math.Min(r, g), b);

            if (max == min)
                return 0;

            return 1 - (min/max);
        }

        /// <summary>
        /// Hue 값을 0~360 사이의 값으로 반환합니다.
        /// </summary>
        /// <param name="color"></param>
        /// <returns></returns>
        static public double GetHue(Color color)
        {
            double r = GetUnitColorAmount(color.R);
            double g = GetUnitColorAmount(color.G);
            double b = GetUnitColorAmount(color.B);

            double max = Math.Max(Math.Max(r, g), b);
            double min = Math.Min(Math.Min(r, g), b);

            if (max == min)
            {
                return 0;
            }
            else if (max == r)
            {
                return (60 * (g - b) / (max - min) + 360) % 360;

            }
            else if (max == color.G)
            {
                return (60 * (b- r) / (max - min) + 120);
            }
            else
            {
                return (60 * (r - g) / (max - min) + 240);
            }
        }
    }
}
